@@ -59,10 +59,10 @@ def sync_project(
         url = hedra_project_config.get('project_url')
     
     if username is None:
-        username = hedra_project_config.get('project_username')
+        username = hedra_project_config.get('project_username', "")
 
     if password is None:
-        password = hedra_project_config.get('project_password')
+        password = hedra_project_config.get('project_password', "")
 
     if branch is None:
         branch = hedra_project_config.get('project_branch', 'main')
@@ -77,20 +77,6 @@ def sync_project(
             '--url'
         )
 
-    if username is None:
-        raise NotSetError(
-            path,
-            'username',
-            '--username'
-        )
-
-    if password is None:
-        raise NotSetError(
-            path,
-            'password',
-            '--password'
-        )
-
     parsed_url = urlparse(url)
     repo_url = f'{parsed_url.scheme}://{username}:{password}@{parsed_url.hostname}{parsed_url.path}'
 
