@@ -12,7 +12,8 @@ class BaseAction:
         'name', 
         'is_setup', 
         'metadata', 
-        'hooks'
+        'hooks',
+        'event'
     )
 
     def __init__(
@@ -26,4 +27,5 @@ class BaseAction:
         self.name = name
         self.is_setup = False
         self.metadata = Metadata(user, tags)
-        self.hooks: Hooks[BaseAction] = Hooks()
\ No newline at end of file
+        self.hooks: Hooks[BaseAction] = Hooks()
+        self.event = None
\ No newline at end of file
