@@ -44,7 +44,11 @@ class MercuryUDPClient:
         self.waiter = None
 
         self.ssl_context = get_default_ssl_context()
-        
+    
+    async def set_pool(self, concurrency: int):
+        self.sem = asyncio.Semaphore(value=concurrency)
+        self.pool = Pool(concurrency, reset_connections=self.pool.reset_connections)
+        self.pool.create_pool()
 
     async def wait_for_active_threshold(self):
         if self.waiter is None:
@@ -125,6 +129,8 @@ class MercuryUDPClient:
         response = UDPResult(action)
         response.wait_start = time.monotonic()
         self.active += 1
+
+        action_event = action.event
  
         async with self.sem:
             connection = self.pool.connections.pop()
@@ -136,8 +142,8 @@ class MercuryUDPClient:
                     action.hooks.channel_events.append(event)
                     await event.wait()
 
-                if action.hooks.before:
-                    action = await action.hooks.before.call(action, response)
+                if action_event:
+                    action, response = await action_event.execute_pre(action, response)
                     action.setup()
 
                 response.start = time.monotonic()
@@ -167,8 +173,8 @@ class MercuryUDPClient:
 
                 self.pool.connections.append(connection)
 
-                if action.hooks.after:
-                    action = await action.hooks.after.call(action, response)
+                if action_event:
+                    action, response = await action_event.execute_post(action, response)
                     action.setup()
 
                 if action.hooks.notify:
