@@ -4,14 +4,16 @@ from .hook_types import (
     before,
     channel,
     check,
+    condition,
     context,
     depends,
     event,
     metric,
-    restore,
+    load,
     save,
     setup,
     task,
     teardown,
+    transform,
     validate
 )
