@@ -3,17 +3,19 @@ from enum import Enum
 
 class HookType(Enum):
     ACTION='ACTION'
-    SETUP='SETUP'
-    TEARDOWN='TEARDOWN'
-    BEFORE='BEFORE'
     AFTER='AFTER'
-    CHECK='CHECK'
-    METRIC='METRIC'
-    VALIDATE='VALIDATE'
-    INTERNAL='INTERNAL'
-    SAVE='SAVE'
-    TASK='TASK'
+    BEFORE='BEFORE'
     CHANNEL='CHANNEL'
-    EVENT='EVENT'
+    CHECK='CHECK'
     CONTEXT='CONTEXT'
-    RESTORE='RESTORE'
+    CONDITION='CONDITION'
+    EVENT='EVENT'
+    INTERNAL='INTERNAL'
+    METRIC='METRIC'
+    LOAD='LOAD'
+    SAVE='SAVE'
+    SETUP='SETUP'
+    TASK='TASK'
+    TEARDOWN='TEARDOWN'
+    TRANSFORM='TRANSFORM'
+    VALIDATE='VALIDATE'
