@@ -39,4 +39,16 @@ class TaskHook(Hook):
             weight=weight,
             order=order,
             **metadata
+        )
+
+    def copy(self):
+        return TaskHook(
+            self.name,
+            self.shortname,
+            self._call,
+            weight=self.metadata.weight,
+            order=self.metadata.order,
+            checks=self.checks,
+            notify=self.notifiers,
+            listen=self.listeners
         )
\ No newline at end of file
