@@ -1,11 +1,12 @@
 from typing import Tuple, Optional
-from pydantic import BaseModel, validator, StrictStr, StrictBool
+from pydantic import BaseModel, validator, StrictStr, StrictBool, StrictInt
 
 
 class EventHookValidator(BaseModel):
     names: Optional[Tuple[StrictStr, ...]]
     pre: StrictBool=False
     key: Optional[StrictStr]
+    order: StrictInt
 
     @validator('names')
     def validate_names(cls, vals):
@@ -14,9 +15,16 @@ class EventHookValidator(BaseModel):
 
 class EventValidator:
 
-    def __init__(__pydantic_self__, *names: Tuple[str, ...], pre: bool=False, key: Optional[str]=None) -> None:
+    def __init__(
+        self, 
+        *names: Tuple[str, ...], 
+        pre: bool=False, 
+        key: Optional[str]=None,
+        order: int=1
+    ) -> None:
         EventHookValidator(
             names=names,
             pre=pre,
-            key=key
+            key=key,
+            order=order
         )
