@@ -1,34 +1,32 @@
 
 import dill
 import time
-import asyncio
 import statistics
 from collections import defaultdict
-from typing import Generic, List, Union
-from hedra.core.engines.client import Client
+from typing import Generic, List, Union, Any, Dict
 from typing_extensions import TypeVarTuple, Unpack
-from hedra.core.graphs.events import Event
+from hedra.core.engines.client import Client
+from hedra.core.engines.client.config import Config
 from hedra.core.engines.types.common.types import RequestTypes
-from hedra.core.engines.types.registry import registered_engines
-from hedra.core.graphs.hooks.registry.registry_types import (
-    ActionHook,
-    EventHook, 
-    CheckHook,
-    ContextHook,
-    TaskHook
-)
-from hedra.core.graphs.hooks.hook_types.hook_type import HookType
-from hedra.core.graphs.hooks.hook_types.internal import Internal
-from hedra.core.graphs.stages.types.stage_types import StageTypes
 from hedra.core.engines.types.playwright import (
     MercuryPlaywrightClient,
     ContextConfig
 )
-from hedra.core.personas.persona_registry import get_persona, registered_personas
-from hedra.plugins.types.plugin_types import PluginType
-
-from hedra.core.graphs.stages.base.parallel.partition_method import PartitionMethod
+from hedra.core.engines.types.registry import registered_engines
+from hedra.core.graphs.hooks.hook_types.condition import condition
+from hedra.core.graphs.hooks.hook_types.context import context
+from hedra.core.graphs.hooks.hook_types.event import event
+from hedra.core.graphs.hooks.hook_types.hook_type import HookType
+from hedra.core.graphs.hooks.hook_types.internal import Internal
+from hedra.core.graphs.hooks.registry.registry_types import (
+    ActionHook,
+    TaskHook
+)
 from hedra.core.graphs.stages.base.stage import Stage
+from hedra.core.graphs.stages.base.parallel.partition_method import PartitionMethod
+from hedra.core.graphs.stages.types.stage_types import StageTypes
+from hedra.core.personas.persona_registry import get_persona, registered_personas, DefaultPersona
+from hedra.plugins.types.plugin_types import PluginType
 from .parallel import execute_actions
 
 
@@ -49,16 +47,18 @@ class Execute(Stage, Generic[Unpack[T]]):
         )
         
         self.accepted_hook_types = [ 
-            HookType.SETUP, 
-            HookType.BEFORE, 
             HookType.ACTION,
-            HookType.TASK,
             HookType.AFTER,
-            HookType.TEARDOWN,
-            HookType.CHECK,
+            HookType.BEFORE, 
             HookType.CHANNEL, 
+            HookType.CHECK,
+            HookType.CONDITION,
+            HookType.CONTEXT,
             HookType.EVENT,
-            HookType.CONTEXT
+            HookType.SETUP, 
+            HookType.TASK,
+            HookType.TEARDOWN,
+            HookType.TRANSFORM
         ]
 
         self.concurrent_pool_aware_stages = 0
@@ -70,28 +70,32 @@ class Execute(Stage, Generic[Unpack[T]]):
 
     @Internal()
     async def run(self):
+        await self.setup_events()
+        await self.dispatcher.dispatch_events()
 
-        events: List[Union[EventHook, Event]] = [event for event in self.hooks[HookType.EVENT]]
-        pre_events: List[EventHook] = [
-            event for event in events if isinstance(event, EventHook) and event.pre
+    @context()
+    async def get_stage_config(
+        self,
+        setup_config: Config=None,
+        setup_by: str=None,
+        execute_hooks: List[Union[ActionHook , TaskHook]] = []
+    ):
+        self.context.ignore_serialization_filters = [
+            'execute_hooks'
         ]
-        
-        if len(pre_events) > 0:
-            pre_event_names = ", ".join([
-                event.shortname for event in pre_events
-            ])
+        persona_type_name = setup_config.persona_type.capitalize()
 
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing PRE events - {pre_event_names}')
-            await asyncio.wait([
-                asyncio.create_task(event.call()) for event in pre_events
-            ], timeout=self.stage_timeout)
+        await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing - {setup_config.batch_size} - VUs over {self.workers} threads for {setup_config.total_time_string} using - {persona_type_name} - persona')
+        await self.logger.spinner.append_message(f'Stage {self.name} executing - {setup_config.batch_size} - VUs over {self.workers} threads for {setup_config.total_time_string} using - {persona_type_name} - persona')
 
-        config = self.client._config
-        persona_type_name = config.persona_type.capitalize()
-
-        await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing - {config.batch_size} - VUs over {self.workers} threads for {config.total_time_string} using - {persona_type_name} - persona')
-        await self.logger.spinner.append_message(f'Stage {self.name} executing - {config.batch_size} - VUs over {self.workers} threads for {config.total_time_string} using - {persona_type_name} - persona')
+        return {
+            'execute_hooks': execute_hooks,
+            'setup_by': setup_by,
+            'execute_stage_config': setup_config
+        }
 
+    @event('get_stage_config')
+    async def get_stage_plugins(self):
         engine_plugins = self.plugins_by_type.get(PluginType.ENGINE)
         for plugin in engine_plugins.values():
             registered_engines[plugin.name] = plugin
@@ -100,58 +104,36 @@ class Execute(Stage, Generic[Unpack[T]]):
         persona_plugins = self.plugins_by_type.get(PluginType.PERSONA)
 
         for plugin_name, plugin in persona_plugins.items():
-            registered_personas[plugin_name] = lambda config: plugin(config)
+            registered_personas[plugin_name] = lambda plugin_config: plugin(plugin_config)
             await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Loaded Persona plugin - {plugin.name}')
 
-        if self.workers > 1:
+        source_stage_plugins = defaultdict(list)
+        for plugin in self.plugins.values():
+            source_stage_plugins[plugin.type].append(plugin.name)
 
-            source_stage_plugins = defaultdict(list)
-            for plugin in self.plugins.values():
-                source_stage_plugins[plugin.type].append(plugin.name)
+        return {
+            'stage_plugins': source_stage_plugins
+        }
 
-            await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Provisioning execution over - {self.workers} - workers')
+    @condition('get_stage_plugins')
+    async def check_has_multiple_workers(self):
+        return {
+            'stage_has_multiple_workers': self.workers > 1
+        }
 
-            action_hooks: List[ActionHook] = self.hooks[HookType.ACTION]
-            hooks = [
-                {
-                    'graph_name': self.graph_name,
-                    'graph_path': self.graph_path,
-                    'stage': hook.stage,
-                    'timeouts': hook.session.timeouts,
-                    'reset_connections': hook.session.pool.reset_connections,
-                    'hook_name': hook.name,
-                    'hook_shortname': hook.shortname,
-                    'hook_type': hook.hook_type,
-                    'stage': hook.stage,
-                    'weight': hook.metadata.weight,
-                    'order': hook.metadata.order,
-                    'checks': [check.__name__ for check in hook.checks],
-                    **hook.action.to_serializable()
-                } for hook in action_hooks
-            ]
+    @event('check_has_multiple_workers')
+    async def run_multiple_worker_jobs(
+        self,
+        stage_has_multiple_workers: bool = False,
+        execute_stage_config: Config=None,
+        stage_plugins: Dict[str, List[Any]]={},
+        setup_by: str=None
+    ):
+        if stage_has_multiple_workers:
+            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Starting execution for - {self.workers} workers')
 
-            task_hooks: List[TaskHook] = self.hooks[HookType.TASK]
+            serializable_context = self.context.as_serializable()       
 
-            hooks.extend([
-                {
-                    'graph_name': self.graph_name,
-                    'graph_path': self.graph_path,
-                    'stage': hook.stage,
-                    'timeouts': hook.session.timeouts,
-                    'reset_connections': False,
-                    'hook_name': hook.name,
-                    'hook_shortname': hook.shortname,
-                    'hook_type': hook.hook_type,
-                    'stage': hook.stage,
-                    'weight': hook.metadata.weight,
-                    'order': hook.metadata.order,
-                    'checks': [check.__name__ for check in hook.checks],
-                    **hook.action.to_serializable()
-                } for hook in task_hooks
-            ])
-
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Starting execution for - {self.workers} workers')             
-    
             results_sets = await self.executor.execute_stage_batch(
                 execute_actions,
                 [
@@ -162,28 +144,39 @@ class Execute(Stage, Generic[Unpack[T]]):
                         'source_stage_name': self.name,
                         'source_stage_linked_events': self.linked_events,
                         'source_stage_context': {
-                            context_key: context_value for context_key, context_value in self.context if context_key not in self.context.known_keys
+                            context_key: context_value for context_key, context_value in serializable_context
                         },
+                        'source_setup_stage_name': setup_by,
                         'source_stage_id': self.stage_id,
-                        'source_stage_plugins': source_stage_plugins,
-                        'source_stage_config': self.client._config,
+                        'source_stage_plugins': stage_plugins,
+                        'source_stage_config': execute_stage_config,
                         'partition_method': PartitionMethod.BATCHES,
                         'workers': self.workers,
-                        'worker_id': idx + 1,
-                        'config': self.client._config,
-                        'hooks': hooks
-                    }) for idx in range(self.executor.max_workers)
+                        'worker_id': idx + 1
+                    }) for idx in range(self.workers)
                 ]
             )
 
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Completed execution for - {self.workers} workers')            
-            
-            results = []
+            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Completed execution for - {self.workers} workers')    
+
+
+            return {
+                'stage_results': results_sets
+            }
+
+    @context('run_multiple_worker_jobs')
+    async def aggregate_multiple_worker_results(
+        self,
+        stage_has_multiple_workers: bool = False,
+        stage_results: List[List[Any]]=[]
+    ):
+        if stage_has_multiple_workers:
+            aggregate_results = []
             elapsed_times = []
             stage_contexts = defaultdict(list)
 
-            for result_set in results_sets:
-                results.extend(result_set.get('results'))
+            for result_set in stage_results:
+                aggregate_results.extend(result_set.get('results'))
                 elapsed_times.append(result_set.get('total_elapsed'))
 
                 pipeline_context = result_set.get('context', {})
@@ -192,14 +185,28 @@ class Execute(Stage, Generic[Unpack[T]]):
 
             self.context[self.name] = stage_contexts
 
-            total_results = len(results)
+            total_results = len(aggregate_results)
             total_elapsed = statistics.median(elapsed_times)
 
-        else:
+            await self.logger.filesystem.aio['hedra.core'].info( f'{self.metadata_string} - Completed - {total_results} actions at  {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
+            await self.logger.spinner.set_default_message(f'Stage - {self.name} completed {total_results} actions at {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
 
-            start = time.monotonic()
+            return {
+                'stage_results': aggregate_results,
+                'total_results': total_results,
+                'total_elapsed': total_elapsed
+            }
 
-            persona_config = self.client._config
+    @event('check_has_multiple_workers')
+    async def setup_single_worker_job(
+        self,
+        stage_has_multiple_workers: bool = False,
+        execute_stage_config: Config=None,
+    ):
+
+        if stage_has_multiple_workers is False:
+
+            persona_config = execute_stage_config
             persona = get_persona(persona_config)
             persona.setup(self.hooks, self.metadata_string)
 
@@ -230,8 +237,22 @@ class Execute(Stage, Generic[Unpack[T]]):
                         color_scheme=persona_config.color_scheme,
                         options=persona_config.playwright_options
                     ))
+            return {
+                'stage_persona': persona
+            }
 
-            results = await persona.execute()
+
+    @context('setup_single_worker_job')
+    async def run_single_worker_job(
+        self,
+        stage_has_multiple_workers: bool = False,
+        stage_persona: DefaultPersona=None
+    ):
+        if stage_has_multiple_workers is False:
+
+            start = time.monotonic()
+
+            results = await stage_persona.execute()
 
             elapsed = time.monotonic() - start
 
@@ -247,34 +268,13 @@ class Execute(Stage, Generic[Unpack[T]]):
             self.context[self.name] = stage_contexts
 
             total_results = len(results)
-            total_elapsed = persona.total_elapsed
+            total_elapsed = stage_persona.total_elapsed
 
-        post_events: List[EventHook] = [
-            event for event in events if isinstance(event, EventHook) and event.pre is False
-        ]
+            await self.logger.filesystem.aio['hedra.core'].info( f'{self.metadata_string} - Completed - {total_results} actions at  {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
+            await self.logger.spinner.set_default_message(f'Stage - {self.name} completed {total_results} actions at {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
 
-        if len(post_events) > 0:
-            post_event_names = ", ".join([
-                event.shortname for event in post_events
-            ])
-
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing POST events - {post_event_names}')
-            await asyncio.wait([
-                asyncio.create_task(event.call()) for event in post_events
-            ], timeout=self.stage_timeout)
-
-
-        await self.logger.filesystem.aio['hedra.core'].info( f'{self.metadata_string} - Completed - {total_results} actions at  {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
-        await self.logger.spinner.set_default_message(f'Stage - {self.name} completed {total_results} actions at {round(total_results/total_elapsed)} actions/second over {round(total_elapsed)} seconds')
-
-        context_hooks: List[ContextHook] = self.hooks[HookType.CONTEXT]
-        context_hooks: List[ContextHook] = self.hooks[HookType.CONTEXT]
-        await asyncio.gather(*[
-            asyncio.create_task(context_hook.call(self.context)) for context_hook in context_hooks
-        ])
-        
-        return {
-            'results': results,
-            'total_results': total_results,
-            'total_elapsed': total_elapsed
-        }
\ No newline at end of file
+            return {
+                'stage_results': results,
+                'total_results': total_results,
+                'total_elapsed': total_elapsed
+            }
\ No newline at end of file
