@@ -1,13 +1,18 @@
 import asyncio
 import psutil
 import traceback
+import inspect
+from collections import defaultdict
 from typing_extensions import TypeVarTuple, Unpack
 from typing import Dict, Generic, List, Any, Union, Coroutine
-from hedra.core.graphs.events import Event
+from hedra.core.graphs.hooks.hook_types.condition import condition
+from hedra.core.graphs.hooks.hook_types.context import context
+from hedra.core.graphs.hooks.hook_types.event import event
+from hedra.core.graphs.hooks.hook_types.transform import transform
 from hedra.core.graphs.hooks.registry.registry_types.hook import Hook
 from hedra.core.graphs.hooks.hook_types.hook_type import HookType
 from hedra.core.graphs.hooks.hook_types.internal import Internal
-from hedra.core.engines.client.client import Client
+from hedra.core.engines.client.client import Client, config_registry
 from hedra.core.engines.client.config import Config
 from hedra.core.engines.types.common.base_action import BaseAction
 from hedra.core.engines.types.task.task import Task
@@ -17,8 +22,6 @@ from hedra.core.graphs.hooks.registry.registry_types import (
     BeforeHook,
     ChannelHook,
     CheckHook,
-    ContextHook,
-    EventHook,
     SetupHook,
     TaskHook
 )
@@ -44,8 +47,9 @@ T = TypeVarTuple('T')
 
 class SetupCall:
 
-    def __init__(self, hook: Hook, retries: int=1) -> None:
+    def __init__(self, hook: Hook, config: Config, retries: int=1) -> None:
         self.hook = hook
+        self.config = config
         self.hook_name = self.hook.hook_type.name.capitalize()
         self.exception = None
         self.action_store = None
@@ -60,6 +64,8 @@ class SetupCall:
 
         for _ in range(self.retries):
             try:
+
+                self.hook.stage_instance.client._config = self.config
                 await self.hook.call()
 
             except Exception as setup_exception:
@@ -106,7 +112,14 @@ class Setup(Stage, Generic[Unpack[T]]):
         super().__init__()
         self.generation_setup_candidates = 0
         self.stages: Dict[str, Stage] = {}
-        self.accepted_hook_types = [ HookType.SETUP, HookType.EVENT, HookType.CONTEXT ]
+        self.accepted_hook_types = [ 
+            HookType.CONDITION,
+            HookType.CONTEXT,
+            HookType.EVENT, 
+            HookType.SETUP,
+            HookType.TRANSFORM 
+        ]
+
         self.persona_types = PersonaTypesMap()
         self.config = Config(
             log_level=self.log_level,
@@ -143,49 +156,60 @@ class Setup(Stage, Generic[Unpack[T]]):
             'get_checks',
             'setup'
         ])
+        
+        self.internal_events = [
+            'collect_target_stages'
+        ]
 
     @Internal()
     async def run(self):
+        await self.setup_events()
+        await self.dispatcher.dispatch_events()
 
-        events: List[Union[EventHook, Event]] = [event for event in self.hooks[HookType.EVENT]]
-        pre_events: List[EventHook] = [
-            event for event in events if isinstance(event, EventHook) and event.pre
+    @Internal()
+    async def run_internal(self):
+        await self.setup_events()
+        
+        initial_events = self.dispatcher.initial_events
+        self.dispatcher.initial_events = [
+            initial_event for initial_event in self.dispatcher.initial_events if initial_event.source.shortname in self.internal_events
         ]
-        
-        if len(pre_events) > 0:
-            pre_event_names = ", ".join([
-                event.shortname for event in pre_events
-            ])
 
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing PRE events - {pre_event_names}')
-            await asyncio.wait([
-                asyncio.create_task(event.call()) for event in pre_events
-            ], timeout=self.stage_timeout)
-        
+        await self.dispatcher.dispatch_events()
+        self.dispatcher.initial_events = initial_events
+    
+    @context()
+    async def collect_target_stages(
+        self,
+        setup_stages: Dict[str, Stage]={},
+        setup_config: Config=None
+    ):
         bypass_connection_validation = self.core_config.get('bypass_connection_validation', False)
         connection_validation_retries = self.core_config.get('connection_validation_retries', 3)
 
         await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Starting setup')
         
-        setup_hooks: List[SetupHook] = self.hooks[HookType.SETUP]
-        setup_hook_names = ', '.join([hook.name for hook in setup_hooks])
-        
-        if len(setup_hooks) > 0:
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Runnning Setup hooks for stage - {setup_hook_names}')
-        
-        results = await asyncio.gather(*[setup_hook.call(self.context) for setup_hook in setup_hooks])
-        for result in results:
-            self.context = result
+        return {
+            'setup_config': setup_config,
+            'setup_stages': setup_stages,
+            'bypass_connection_validation': bypass_connection_validation,
+            'connection_validation_retries': connection_validation_retries
+        }
 
-        execute_stage_id = 1
-
-        stages = dict(self.stages)
-
-        execute_stage_names = ', '.join(list(stages.keys()))
+    @event('collect_target_stages')
+    async def configure_target_stages(
+        self, 
+        setup_stages: Dict[str, Stage]={},
+        setup_config: Config=None
+    ):
+    
+        execute_stage_names = ', '.join(list(setup_stages.keys()))
 
         await self.logger.spinner.append_message(f'Setting up - {execute_stage_names}')
+
+        execute_stage_id = 1
         
-        for execute_stage_name, execute_stage in stages.items():
+        for execute_stage_name, execute_stage in setup_stages.items():
 
             execute_stage.execution_stage_id = execute_stage_id
             execute_stage.execute_setup_stage = self.name
@@ -202,50 +226,77 @@ class Setup(Stage, Generic[Unpack[T]]):
                 
                 await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Loaded Persona plugin - {plugin.name} - for Execute stae - {execute_stage_name}')
    
-            client = Client(
-                self.graph_name,
-                self.graph_id,
-                execute_stage.name,
-                execute_stage.stage_id
-            )
-            await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Created Client, id - {client.client_id} - for Execute stage - {execute_stage_name}')
-
             engine_plugins: Dict[str, EnginePlugin] = self.plugins_by_type[PluginType.ENGINE]
 
             for plugin_name, plugin in engine_plugins.items():
-                client.plugin[plugin_name] = plugin(self.config)
+                execute_stage.client.plugin[plugin_name] = plugin(setup_config)
                 plugin.name = plugin_name
                 execute_stage.plugins[plugin_name] = plugin
                 self.plugins_by_type[plugin_name] = plugin
 
                 await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Loaded Engine plugin - {plugin.name} - for Execute stage - {execute_stage_name}')
 
+            execute_stage.client._config = setup_config
+            setup_stages[execute_stage_name] = execute_stage
 
-            execute_stage.client = client
-            execute_stage.client._config = self.config
+        return  {
+            'prepared_stages': setup_stages
+        }
 
-            action_hooks: List[ActionHook] = execute_stage.hooks[HookType.ACTION]
-            for hook in action_hooks:
+    @event('configure_target_stages')
+    async def collect_action_hooks(
+        self,
+        prepared_stages: Dict[str, Stage]={}
+    ):
+    
+        actions: List[ActionHook] = []
+        for execute_stage in prepared_stages.values():
+            actions.extend(execute_stage.hooks[HookType.ACTION])
 
-                execute_stage.client.next_name = hook.name
-                execute_stage.client.intercept = True
+        return {
+            'actions': actions
+        }
 
-                await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Client intercept set to {execute_stage.client.intercept} - Action calls for client id - {execute_stage.client.client_id} - will be suspended on execution')
+    
+    @condition('collect_action_hooks')
+    async def check_actions_setup_needed(self, actions: List[ActionHook]=[]):
+        return {
+            'has_actions_for_setup': len(actions) > 0
+        }
+        
+    @transform('check_actions_setup_needed')
+    async def setup_action(
+        self,
+        actions: ActionHook=None,
+        has_actions_for_setup: bool = False,
+        bypass_connection_validation: bool=False,
+        connection_validation_retries: int=3,
+        setup_config: Config=None
+    ):
+            if has_actions_for_setup:
+                hook = actions
+                hook.stage_instance.client.next_name = hook.name
+                hook.stage_instance.client.intercept = True
+                hook.stage_instance.client._config = setup_config
+
+                execute_stage_name = hook.stage_instance.name
+
+                await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Client intercept set to {hook.stage_instance.client.intercept} - Action calls for client id - {hook.stage_instance.client.client_id} - will be suspended on execution')
 
                 await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Setting up Action - {hook.name}:{hook.hook_id} - for Execute stage - {execute_stage_name}')
                 await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Preparing Action hook - {hook.name}:{hook.hook_id} - for suspension - Execute stage - {execute_stage_name}')
 
-                execute_stage.client.actions.set_waiter(execute_stage.name)
+                hook.stage_instance.client.actions.set_waiter(hook.stage_instance.name)
 
-                setup_call = SetupCall(hook, retries=connection_validation_retries)
+                setup_call = SetupCall(hook, setup_config, retries=connection_validation_retries)
 
                 setup_call.metadata_string = self.metadata_string
-                setup_call.action_store = execute_stage.client.actions
+                setup_call.action_store = hook.stage_instance.client.actions
 
                 await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Executing Action hook call - {hook.name}:{hook.hook_id} - Execute stage - {execute_stage_name}')
 
                 task = asyncio.create_task(setup_call.setup())
-                await execute_stage.client.actions.wait_for_ready(setup_call)   
+                await hook.stage_instance.client.actions.wait_for_ready(setup_call)   
 
                 await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Exiting suspension for Action - {hook.name}:{hook.hook_id} - Execute stage - {execute_stage_name}')
 
@@ -264,11 +315,11 @@ class Setup(Stage, Generic[Unpack[T]]):
 
                     if bypass_connection_validation:
 
-                        hook.hook_type = HookType.TASK
+                        action.hook_type = HookType.TASK
 
-                        execute_stage.hooks[HookType.TASK].append(hook)
-                        action_idx = execute_stage.hooks[HookType.ACTION].index(hook)
-                        execute_stage.hooks[HookType.ACTION].pop(action_idx)
+                        hook.stage_instance.hooks[HookType.TASK].append(hook)
+                        action_idx = hook.stage_instance.hooks[HookType.ACTION].index(hook)
+                        hook.stage_instance.hooks[HookType.ACTION].pop(action_idx)
 
                     else:
                         raise hook_setup_exception
@@ -281,86 +332,129 @@ class Setup(Stage, Generic[Unpack[T]]):
 
                 except asyncio.TimeoutError:
                     pass
-
-                if hook.hook_type == HookType.ACTION:
-
-                    action, session = execute_stage.client.actions.get(
-                        execute_stage.name,
-                        hook.name
-                    )
-
-                    await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Successfully retrieved prepared Action and Session for action - {action.name}:{action.action_id} - Execute stage - {execute_stage_name}')
-                    
-                    action.hooks.before =  await self.get_hook(execute_stage, hook.shortname, HookType.BEFORE)
-                    action.hooks.after = await self.get_hook(execute_stage, hook.shortname, HookType.AFTER)
-                    action.hooks.checks = await self.get_checks(execute_stage, hook.shortname)
-                    
-                    hook.session = session
-                    hook.action = action    
-
-            task_hooks: List[TaskHook] = execute_stage.hooks[HookType.TASK]
-            for hook in task_hooks:
-
-                await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Loading Task hook - {hook.name}:{hook.hook_id} - to Execute stage - {execute_stage_name}')
-
-                execute_stage.client.next_name = hook.name
-                task, session = execute_stage.client.task.call(
-                    hook.call,
-                    env=hook.metadata.env,
-                    user=hook.metadata.user,
-                    tags=hook.metadata.tags
+                
+                action, session = hook.stage_instance.client.actions.get(
+                    hook.stage_instance.name,
+                    hook.name
                 )
 
-                await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Successfully retrieved task and session for Task - {task.name}:{task.action_id} - Execute stage - {execute_stage_name}')
-
-                task.hooks.checks = await self.get_checks(execute_stage, hook.shortname)
+                await session.set_pool(setup_config.batch_size)
 
+                await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Successfully retrieved prepared Action and Session for action - {hook.name}:{action.action_id} - Execute stage - {execute_stage_name}')
+                
+                action.hooks.before =  await self.get_hook(hook.stage_instance, hook.shortname, HookType.BEFORE)
+                action.hooks.after = await self.get_hook(hook.stage_instance, hook.shortname, HookType.AFTER)
+                action.hooks.checks = await self.get_checks(hook.stage_instance, hook.shortname)
+                
                 hook.session = session
-                hook.action = task  
+                hook.action = action  
 
+                return {
+                    'actions': actions
+                }
+
+    @event('setup_action')
+    async def collect_task_hooks(
+        self,
+        setup_stages: Dict[str, Stage]={}
+    ):
+        tasks: List[TaskHook] = []
+        for execute_stage in setup_stages.values():
+            tasks.extend(execute_stage.hooks[HookType.TASK])
+
+        return {
+            'tasks': tasks
+        }
+    
+    @condition('collect_task_hooks')
+    async def check_tasks_setup_needed(self, tasks: List[ActionHook]=[]):
+        return {
+            'has_tasks_for_setup': len(tasks) > 0
+        }
+
+    @transform('check_tasks_setup_needed')
+    async def setup_task(
+        self,
+        tasks: TaskHook=None,
+        has_tasks_for_setup: bool=False,
+        setup_config: Config=None
+    ):
+        if has_tasks_for_setup:
+            hook = tasks
+            execute_stage: Stage = hook.stage_instance
+            execute_stage.client.next_name = hook.name
+            execute_stage.client.intercept = True
+            execute_stage_name = execute_stage.name
+
+            await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Loading Task hook - {hook.name}:{hook.hook_id} - to Execute stage - {execute_stage_name}')
+
+            execute_stage.client.next_name = hook.name
+            task, session = execute_stage.client.task.call(
+                hook.call,
+                env=hook.metadata.env,
+                user=hook.metadata.user,
+                tags=hook.metadata.tags
+            )
+            
+            await session.set_pool(setup_config.batch_size)
+
+            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Successfully retrieved task and session for Task - {hook.name}:{task.action_id} - Execute stage - {execute_stage_name}')
+
+            task.hooks.checks = await self.get_checks(execute_stage, hook.shortname)
+
+            hook.session = session
+            hook.action = task  
+
+            return {
+                'tasks': tasks
+            }
+
+    @context('setup_task')
+    async def apply_channels(
+        self,
+        setup_stages: Dict[str, Stage]=[],
+        actions: List[ActionHook]=[],
+        tasks: List[TaskHook]=[],
+        setup_config: Config=None
+    ):
+        actions_by_stage = defaultdict(list)
+        tasks_by_stage = defaultdict(list)
+
+        for action in actions:
+            actions_by_stage[action.stage].append(action)
+
+        for task in tasks:
+            tasks_by_stage[task.stage].append(task)
+
+        for execute_stage in setup_stages.values():
             await self.get_channels(execute_stage)
 
             execute_stage.client.intercept = False
+            execute_stage.hooks[HookType.ACTION] = actions_by_stage[execute_stage.name]
+            execute_stage.hooks[HookType.TASK] = tasks_by_stage[execute_stage.name]
+            execute_stage.context['execute_hooks'] = [
+                *actions_by_stage[execute_stage.name],
+                *tasks_by_stage[execute_stage.name]
+            ]
+
             await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Client intercept set to {execute_stage.client.intercept} - Action calls for client id - {execute_stage.client.client_id} - will not be suspended on execution')
 
-            for setup_hook in execute_stage.hooks[HookType.SETUP]:
-                await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing Setup hook - {setup_hook.name} - for Execute stage - {execute_stage_name}')
-
-                execute_stage.context = await setup_hook.call(self.context)
-
-            self.stages[execute_stage_name] = execute_stage
+            self.stages[execute_stage.name] = execute_stage
 
             actions_generated_count = len(execute_stage.hooks[HookType.ACTION])
+            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Generated - {actions_generated_count} - Actions for Execute stage - {execute_stage.name}')
+
             tasks_generated_count = len(execute_stage.hooks[HookType.TASK])
+            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Generated - {tasks_generated_count} - Tasks for Execute stage - {execute_stage.name}')
+    
+        return {
+            'setup_config': setup_config,
+            'ready_stages': setup_stages
+        }
 
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Generated - {actions_generated_count} - Actions for Execute stage - {execute_stage_name}')
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Generated - {tasks_generated_count} - Tasks for Execute stage - {execute_stage_name}')
-
-        post_events: List[EventHook] = [
-            event for event in events if isinstance(event, EventHook) and event.pre is False
-        ]
-
-        if len(post_events) > 0:
-            post_event_names = ", ".join([
-                event.shortname for event in post_events
-            ])
-
-            await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Executing POST events - {post_event_names}')
-            await asyncio.wait([
-                asyncio.create_task(event.call()) for event in post_events
-            ], timeout=self.stage_timeout)
-
-
-        await self.logger.spinner.set_default_message(f'Setup for - {execute_stage_names} - complete')
-        await self.logger.filesystem.aio['hedra.core'].info(f'{self.metadata_string} - Completed setup')
-
-        context_hooks: List[ContextHook] = self.hooks[HookType.CONTEXT]
-        context_hooks: List[ContextHook] = self.hooks[HookType.CONTEXT]
-        await asyncio.gather(*[
-            asyncio.create_task(context_hook.call(self.context)) for context_hook in context_hooks
-        ])
-
-        return self.stages
+    @event('apply_channels')
+    async def complete(self):
+        return {}
 
     @Internal()
     async def get_hook(self, execute_stage: Stage, shortname: str, hook_type: str) -> Coroutine:
