@@ -1,6 +1,7 @@
 import os
+from typing import Union
 from pathlib import Path
-from hedra.core.graphs.events.event import Event
+from hedra.core.graphs.events.base_event import BaseEvent
 from hedra.core.graphs.hooks.hook_types.hook_type import HookType
 from hedra.core.graphs.hooks.registry.registry_types import SaveHook
 from hedra.core.graphs.stages.validate.exceptions import HookValidationError
@@ -15,12 +16,12 @@ class SaveHookValidator(BaseHookVaidator):
             self
         ).__init__(metadata_string)
 
-    async def validate(self, hook: SaveHook):
+    async def validate(self, hook: Union[SaveHook, BaseEvent]):
 
         try:
 
             call = hook._call
-            if isinstance(hook, Event):
+            if isinstance(hook, BaseEvent):
                 call = hook.target._call
 
             await self.logger.filesystem.aio['hedra.core'].debug(f'{self.metadata_string} - Validating {hook.hook_type.name.capitalize()} Hook - {hook.name}:{hook.hook_id}:{hook.hook_id}')
@@ -34,9 +35,6 @@ class SaveHookValidator(BaseHookVaidator):
             if hook.save_path:
                 hook_path_dir = str(Path(hook.save_path).parent.resolve())
                 
-                assert call.__code__.co_argcount > 1, f"Missing required argument 'data' for @save hook {hook.name}:{hook.hook_id}"
-                assert call.__code__.co_argcount < 3, f"Too many args. - @save hook {hook.name}:{hook.hook_id} only requires 'data' as additional args."
-                assert hook.context_key is not None, f"Missing required keyword arg. - @save hook {hook.name}:{hook.hook_id} requires a valid string for 'key'"
                 assert hook.save_path is not None, f"Missing required keyword arg. - @save hook {hook.name}:{hook.hook_id} requires a valid string for 'checkpoint_filepath'"
                 assert isinstance(hook.save_path, str), f"Invalid path type - @save hook {hook.name}:{hook.hook_id} path must be a valid string."
                 assert os.path.exists(hook_path_dir), f"Invalid path - @save hook {hook.name}:{hook.hook_id} path {hook_path_dir} must exist."
