@@ -1,7 +1,6 @@
-from typing import Any, Tuple, Dict
-from hedra.core.graphs.hooks.hook_types.hook_type import HookType
+from typing import Any
 from hedra.core.engines.types.playwright import PlaywrightResult
-from .base_event import BaseEvent
+from .base_result import BaseEvent
 
 
 class PlaywrightEvent(BaseEvent):
