@@ -1,7 +1,7 @@
 from typing import Any, Tuple, Dict
 from hedra.core.graphs.hooks.hook_types.hook_type import HookType
 from hedra.core.engines.types.task.result import TaskResult
-from .base_event import BaseEvent
+from .base_result import BaseEvent
 
 
 class TaskEvent(BaseEvent):
