@@ -3,7 +3,7 @@ import uuid
 from typing import List
 
 from numpy import float32, float64, int16, int32, int64
-from hedra.reporting.events.types.base_event import BaseEvent
+from hedra.reporting.events.types.base_result import BaseEvent
 from hedra.reporting.metric import MetricsSet
 
 
